@@ -5,4 +5,6 @@
 """
 
 from ._device import BaseDevice as Device
-from ._usbmux import Usbmux
\ No newline at end of file
+from ._usbmux import Usbmux
+from ._perf import Performance, DataType
+from .exceptions import *
\ No newline at end of file
